
from abc import ABC, abstractmethod

#TODO: [SCRUM-17] S3Adapter - Future Implementation of BaseFileFormaters to Adapters
class basefileformater(ABC):
  def __init__(self, formater_name='BaseFormater'):
    pass